@@ -1,3 +1,5 @@
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,7 +13,8 @@ namespace mozilla {
 
 namespace ipc {
 class BackgroundParentImpl;
-}
+class PrincipalInfo;
+} // namespace ipc
 
 namespace dom {
 
@@ -21,16 +24,13 @@ class BroadcastChannelParent final : public PBroadcastChannelParent
 {
   friend class mozilla::ipc::BackgroundParentImpl;
 
+  typedef mozilla::ipc::PrincipalInfo PrincipalInfo;
+
 public:
-  void CheckAndDeliver(const ClonedMessageData& aData,
-                       const nsString& aOrigin,
-                       const nsString& aChannel,
-                       bool aPrivateBrowsing);
+  void Deliver(const ClonedMessageData& aData);
 
 private:
-  BroadcastChannelParent(const nsAString& aOrigin,
-                         const nsAString& aChannel,
-                         bool aPrivateBrowsing);
+  explicit BroadcastChannelParent(const nsAString& aOriginChannelKey);
   ~BroadcastChannelParent();
 
   virtual bool
@@ -40,13 +40,11 @@ private:
 
   virtual void ActorDestroy(ActorDestroyReason aWhy) override;
 
-  nsRefPtr<BroadcastChannelService> mService;
-  nsString mOrigin;
-  nsString mChannel;
-  bool mPrivateBrowsing;
+  RefPtr<BroadcastChannelService> mService;
+  const nsString mOriginChannelKey;
 };
 
-} // dom namespace
-} // mozilla namespace
+} // namespace dom
+} // namespace mozilla
 
 #endif // mozilla_dom_BroadcastChannelParent_h
