@@ -4,7 +4,7 @@
  */
 "use strict";
 
-let disableWorkerTest = "This test uses SpecialPowers";
+var disableWorkerTest = "This test uses SpecialPowers";
 
 var self = this;
 
@@ -164,7 +164,7 @@ function testSteps()
     setLowDiskMode(true);
 
     let request = indexedDB.open(dbName, dbVersion + 2);
-    request.onerror = expectedErrorHandler("QuotaExceededError");
+    request.onerror = errorHandler;
     request.onupgradeneeded = grabEventAndContinueHandler;
     request.onsuccess = unexpectedSuccessHandler;
 
@@ -175,12 +175,20 @@ function testSteps()
     let db = event.target.result;
     db.onerror = errorHandler;
 
+    let txn = event.target.transaction;
+    txn.onerror = expectedErrorHandler("AbortError");
+    txn.onabort = grabEventAndContinueHandler;
+
     let objectStore = db.createObjectStore(objectStoreName, objectStoreOptions);
 
     request.onupgradeneeded = unexpectedSuccessHandler;
     event = yield undefined;
 
-    is(event.type, "error", "Failed database upgrade");
+    is(event.type, "abort", "Got correct event type");
+    is(event.target.error.name, "QuotaExceededError", "Got correct error type");
+
+    request.onerror = expectedErrorHandler("AbortError");
+    event = yield undefined;
   }
 
   { // Make sure creating indexes in low disk mode fails.
@@ -214,7 +222,7 @@ function testSteps()
     setLowDiskMode(true);
 
     request = indexedDB.open(dbName, dbVersion + 3);
-    request.onerror = expectedErrorHandler("QuotaExceededError");
+    request.onerror = errorHandler;
     request.onupgradeneeded = grabEventAndContinueHandler;
     request.onsuccess = unexpectedSuccessHandler;
     event = yield undefined;
@@ -223,6 +231,9 @@ function testSteps()
 
     db = event.target.result;
     db.onerror = errorHandler;
+    let txn = event.target.transaction;
+    txn.onerror = expectedErrorHandler("AbortError");
+    txn.onabort = grabEventAndContinueHandler;
 
     objectStore = event.target.transaction.objectStore(objectStoreName);
     let index = objectStore.createIndex(indexName, indexName, indexOptions);
@@ -230,7 +241,11 @@ function testSteps()
     request.onupgradeneeded = unexpectedSuccessHandler;
     event = yield undefined;
 
-    is(event.type, "error", "Failed database upgrade");
+    is(event.type, "abort", "Got correct event type");
+    is(event.target.error.name, "QuotaExceededError", "Got correct error type");
+
+    request.onerror = expectedErrorHandler("AbortError");
+    event = yield undefined;
   }
 
   { // Make sure deleting indexes in low disk mode succeeds.
