@@ -8,6 +8,7 @@
 
 #ifdef XP_WIN
 #include "WMFDecoderModule.h"
+
 #endif
 #ifdef MOZ_FFVPX
 #include "FFVPXRuntimeLinker.h"
@@ -366,7 +367,7 @@ PDMFactory::CreatePDMs()
   }
 #endif
 #ifdef XP_WIN
-  if (MediaPrefs::PDMWMFEnabled() && IsVistaOrLater()) {
+  if (MediaPrefs::PDMWMFEnabled() && IsVistaOrLater() && !IsWin7AndPre2000Compatible()) {
     // *Only* use WMF on Vista and later, as if IceCat is run in Windows 95
     // compatibility mode on Windows 7 (it does happen!) we may crash trying
     // to startup WMF. So we need to detect the OS version here, as in
@@ -374,6 +375,8 @@ PDMFactory::CreatePDMs()
     // the emulated version of Windows. See bug 1279171.
     // Additionally, we don't want to start the RemoteDecoderModule if we
     // expect it's not going to work (i.e. on Windows older than Vista).
+    // IsWin7AndPre2000Compatible() uses GetVersionEx as the user specified OS version can
+    // be reflected when compatibility mode is in effect.
     m = new WMFDecoderModule();
     RefPtr<PlatformDecoderModule> remote = new dom::RemoteDecoderModule(m);
     StartupPDM(remote);
