@@ -205,12 +205,13 @@ LoadDLLs()
 HRESULT
 MFStartup()
 {
-  if (!IsVistaOrLater()) {
+  if (!IsVistaOrLater() || IsWin7AndPre2000Compatible()) {
     // *Only* use WMF on Vista and later, as if IceCat is run in Windows 95
     // compatibility mode on Windows 7 (it does happen!) we may crash trying
     // to startup WMF. So we need to detect the OS version here, as in
     // compatibility mode IsVistaOrLater() and friends behave as if we're on
     // the emulated version of Windows. See bug 1279171.
+    // Using GetVersionEx API which takes compatibility mode into account.
     return E_FAIL;
   }
 
