@@ -10,15 +10,15 @@
 #include "ipc/IPCMessageUtils.h"
 #include "base/message_loop.h"
 
-#include "mozilla/ipc/MessageChannel.h"
 #include "mozilla/ipc/CrossProcessMutex.h"
+#include "mozilla/ipc/MessageChannel.h"
+#include "mozilla/ipc/ProtocolUtils.h"
 #include "mozilla/UniquePtr.h"
 #include "gfxipc/ShadowLayerUtils.h"
 
 #include "npapi.h"
 #include "npruntime.h"
 #include "npfunctions.h"
-#include "nsAutoPtr.h"
 #include "nsString.h"
 #include "nsTArray.h"
 #include "mozilla/Logging.h"
@@ -45,8 +45,8 @@ enum ScriptableObjectType
 };
 
 mozilla::ipc::RacyInterruptPolicy
-MediateRace(const mozilla::ipc::MessageChannel::Message& parent,
-            const mozilla::ipc::MessageChannel::Message& child);
+MediateRace(const mozilla::ipc::MessageChannel::MessageInfo& parent,
+            const mozilla::ipc::MessageChannel::MessageInfo& child);
 
 std::string
 MungePluginDsoPath(const std::string& path);
@@ -94,16 +94,27 @@ struct NPRemoteWindow
   VisualID visualID;
   Colormap colormap;
 #endif /* XP_UNIX */
-#if defined(XP_MACOSX)
+#if defined(XP_MACOSX) || defined(XP_WIN)
   double contentsScaleFactor;
 #endif
 };
 
+// This struct is like NPAudioDeviceChangeDetails, only it uses a
+// std::wstring instead of a const wchar_t* for the defaultDevice.
+// This gives us the necessary memory-ownership semantics without
+// requiring C++ objects in npapi.h.
+struct NPAudioDeviceChangeDetailsIPC
+{
+  int32_t flow;
+  int32_t role;
+  std::wstring defaultDevice;
+};
+
 #ifdef XP_WIN
 typedef HWND NativeWindowHandle;
 #elif defined(MOZ_X11)
 typedef XID NativeWindowHandle;
-#elif defined(XP_DARWIN) || defined(ANDROID) || defined(MOZ_WIDGET_QT)
+#elif defined(XP_DARWIN) || defined(ANDROID)
 typedef intptr_t NativeWindowHandle; // never actually used, will always be 0
 #else
 #error Need NativeWindowHandle for this platform
@@ -120,7 +131,7 @@ typedef mozilla::null_t DXGISharedSurfaceHandle;
 // XXX maybe not the best place for these. better one?
 
 #define VARSTR(v_)  case v_: return #v_
-inline const char* const
+inline const char*
 NPPVariableToString(NPPVariable aVar)
 {
     switch (aVar) {
@@ -153,6 +164,10 @@ NPPVariableToString(NPPVariable aVar)
         VARSTR(NPPVpluginEventModel);
 #endif
 
+#ifdef XP_WIN
+        VARSTR(NPPVpluginRequiresAudioDeviceChanges);
+#endif
+
     default: return "???";
     }
 }
@@ -183,6 +198,10 @@ NPNVariableToString(NPNVariable aVar)
         VARSTR(NPNVprivateModeBool);
         VARSTR(NPNVdocumentOrigin);
 
+#ifdef XP_WIN
+        VARSTR(NPNVaudioDeviceChangeDetails);
+#endif
+
     default: return "???";
     }
 }
@@ -235,9 +254,7 @@ inline nsCString
 NullableString(const char* aString)
 {
     if (!aString) {
-        nsCString str;
-        str.SetIsVoid(true);
-        return str;
+        return NullCString();
     }
     return nsCString(aString);
 }
@@ -280,7 +297,7 @@ struct ParamTraits<NPRect>
     WriteParam(aMsg, aParam.right);
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     uint16_t top, left, bottom, right;
     if (ReadParam(aMsg, aIter, &top) &&
@@ -313,7 +330,7 @@ struct ParamTraits<NPWindowType>
     aMsg->WriteInt16(int16_t(aParam));
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     int16_t result;
     if (aMsg->ReadInt16(aIter, &result)) {
@@ -347,12 +364,12 @@ struct ParamTraits<mozilla::plugins::NPRemoteWindow>
     aMsg->WriteULong(aParam.visualID);
     aMsg->WriteULong(aParam.colormap);
 #endif
-#if defined(XP_MACOSX)
+#if defined(XP_MACOSX) || defined(XP_WIN)
     aMsg->WriteDouble(aParam.contentsScaleFactor);
 #endif
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     uint64_t window;
     int32_t x, y;
@@ -376,7 +393,7 @@ struct ParamTraits<mozilla::plugins::NPRemoteWindow>
       return false;
 #endif
 
-#if defined(XP_MACOSX)
+#if defined(XP_MACOSX) || defined(XP_WIN)
     double contentsScaleFactor;
     if (!aMsg->ReadDouble(aIter, &contentsScaleFactor))
       return false;
@@ -393,7 +410,7 @@ struct ParamTraits<mozilla::plugins::NPRemoteWindow>
     aResult->visualID = visualID;
     aResult->colormap = colormap;
 #endif
-#if defined(XP_MACOSX)
+#if defined(XP_MACOSX) || defined(XP_WIN)
     aResult->contentsScaleFactor = contentsScaleFactor;
 #endif
     return true;
@@ -408,44 +425,6 @@ struct ParamTraits<mozilla::plugins::NPRemoteWindow>
   }
 };
 
-template <>
-struct ParamTraits<NPString>
-{
-  typedef NPString paramType;
-
-  static void Write(Message* aMsg, const paramType& aParam)
-  {
-    WriteParam(aMsg, aParam.UTF8Length);
-    aMsg->WriteBytes(aParam.UTF8Characters,
-                     aParam.UTF8Length * sizeof(NPUTF8));
-  }
-
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
-  {
-    if (ReadParam(aMsg, aIter, &aResult->UTF8Length)) {
-      int byteCount = aResult->UTF8Length * sizeof(NPUTF8);
-      if (!byteCount) {
-        aResult->UTF8Characters = "\0";
-        return true;
-      }
-
-      const char* messageBuffer = nullptr;
-      mozilla::UniquePtr<char[]> newBuffer(new char[byteCount]);
-      if (newBuffer && aMsg->ReadBytes(aIter, &messageBuffer, byteCount )) {
-        memcpy((void*)messageBuffer, newBuffer.get(), byteCount);
-        aResult->UTF8Characters = newBuffer.release();
-        return true;
-      }
-    }
-    return false;
-  }
-
-  static void Log(const paramType& aParam, std::wstring* aLog)
-  {
-    aLog->append(StringPrintf(L"%s", aParam.UTF8Characters));
-  }
-};
-
 #ifdef XP_MACOSX
 template <>
 struct ParamTraits<NPNSString*>
@@ -481,7 +460,7 @@ struct ParamTraits<NPNSString*>
     }
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     bool haveString = false;
     if (!aMsg->ReadBool(aIter, &haveString)) {
@@ -497,15 +476,19 @@ struct ParamTraits<NPNSString*>
       return false;
     }
 
-    UniChar* buffer = nullptr;
+    // Avoid integer multiplication overflow.
+    if (length > INT_MAX / static_cast<long>(sizeof(UniChar))) {
+      return false;
+    }
+
+    auto chars = mozilla::MakeUnique<UniChar[]>(length);
     if (length != 0) {
-      if (!aMsg->ReadBytes(aIter, (const char**)&buffer, length * sizeof(UniChar)) ||
-          !buffer) {
+      if (!aMsg->ReadBytesInto(aIter, chars.get(), length * sizeof(UniChar))) {
         return false;
       }
     }
 
-    *aResult = (NPNSString*)::CFStringCreateWithBytes(kCFAllocatorDefault, (UInt8*)buffer,
+    *aResult = (NPNSString*)::CFStringCreateWithBytes(kCFAllocatorDefault, (UInt8*)chars.get(),
                                                       length * sizeof(UniChar),
                                                       kCFStringEncodingUTF16, false);
     if (!*aResult) {
@@ -545,7 +528,7 @@ struct ParamTraits<NSCursorInfo>
     free(buffer);
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     NSCursorInfo::Type type;
     if (!aMsg->ReadInt(aIter, (int*)&type)) {
@@ -563,16 +546,16 @@ struct ParamTraits<NSCursorInfo>
       return false;
     }
 
-    uint8_t* data = nullptr;
+    auto data = mozilla::MakeUnique<uint8_t[]>(dataLength);
     if (dataLength != 0) {
-      if (!aMsg->ReadBytes(aIter, (const char**)&data, dataLength) || !data) {
+      if (!aMsg->ReadBytesInto(aIter, data.get(), dataLength)) {
         return false;
       }
     }
 
     aResult->SetType(type);
     aResult->SetHotSpot(nsPoint(hotSpotX, hotSpotY));
-    aResult->SetCustomImageData(data, dataLength);
+    aResult->SetCustomImageData(data.get(), dataLength);
 
     return true;
   }
@@ -604,149 +587,13 @@ struct ParamTraits<NSCursorInfo>
   static void Write(Message* aMsg, const paramType& aParam) {
     NS_RUNTIMEABORT("NSCursorInfo isn't meaningful on this platform");
   }
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult) {
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult) {
     NS_RUNTIMEABORT("NSCursorInfo isn't meaningful on this platform");
     return false;
   }
 };
 #endif // #ifdef XP_MACOSX
 
-template <>
-struct ParamTraits<NPVariant>
-{
-  typedef NPVariant paramType;
-
-  static void Write(Message* aMsg, const paramType& aParam)
-  {
-    if (NPVARIANT_IS_VOID(aParam)) {
-      aMsg->WriteInt(0);
-      return;
-    }
-
-    if (NPVARIANT_IS_NULL(aParam)) {
-      aMsg->WriteInt(1);
-      return;
-    }
-
-    if (NPVARIANT_IS_BOOLEAN(aParam)) {
-      aMsg->WriteInt(2);
-      WriteParam(aMsg, NPVARIANT_TO_BOOLEAN(aParam));
-      return;
-    }
-
-    if (NPVARIANT_IS_INT32(aParam)) {
-      aMsg->WriteInt(3);
-      WriteParam(aMsg, NPVARIANT_TO_INT32(aParam));
-      return;
-    }
-
-    if (NPVARIANT_IS_DOUBLE(aParam)) {
-      aMsg->WriteInt(4);
-      WriteParam(aMsg, NPVARIANT_TO_DOUBLE(aParam));
-      return;
-    }
-
-    if (NPVARIANT_IS_STRING(aParam)) {
-      aMsg->WriteInt(5);
-      WriteParam(aMsg, NPVARIANT_TO_STRING(aParam));
-      return;
-    }
-
-    NS_ERROR("Unsupported type!");
-  }
-
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
-  {
-    int type;
-    if (!aMsg->ReadInt(aIter, &type)) {
-      return false;
-    }
-
-    switch (type) {
-      case 0:
-        VOID_TO_NPVARIANT(*aResult);
-        return true;
-
-      case 1:
-        NULL_TO_NPVARIANT(*aResult);
-        return true;
-
-      case 2: {
-        bool value;
-        if (ReadParam(aMsg, aIter, &value)) {
-          BOOLEAN_TO_NPVARIANT(value, *aResult);
-          return true;
-        }
-      } break;
-
-      case 3: {
-        int32_t value;
-        if (ReadParam(aMsg, aIter, &value)) {
-          INT32_TO_NPVARIANT(value, *aResult);
-          return true;
-        }
-      } break;
-
-      case 4: {
-        double value;
-        if (ReadParam(aMsg, aIter, &value)) {
-          DOUBLE_TO_NPVARIANT(value, *aResult);
-          return true;
-        }
-      } break;
-
-      case 5: {
-        NPString value;
-        if (ReadParam(aMsg, aIter, &value)) {
-          STRINGN_TO_NPVARIANT(value.UTF8Characters, value.UTF8Length,
-                               *aResult);
-          return true;
-        }
-      } break;
-
-      default:
-        NS_ERROR("Unsupported type!");
-    }
-
-    return false;
-  }
-
-  static void Log(const paramType& aParam, std::wstring* aLog)
-  {
-    if (NPVARIANT_IS_VOID(aParam)) {
-      aLog->append(L"[void]");
-      return;
-    }
-
-    if (NPVARIANT_IS_NULL(aParam)) {
-      aLog->append(L"[null]");
-      return;
-    }
-
-    if (NPVARIANT_IS_BOOLEAN(aParam)) {
-      LogParam(NPVARIANT_TO_BOOLEAN(aParam), aLog);
-      return;
-    }
-
-    if (NPVARIANT_IS_INT32(aParam)) {
-      LogParam(NPVARIANT_TO_INT32(aParam), aLog);
-      return;
-    }
-
-    if (NPVARIANT_IS_DOUBLE(aParam)) {
-      LogParam(NPVARIANT_TO_DOUBLE(aParam), aLog);
-      return;
-    }
-
-    if (NPVARIANT_IS_STRING(aParam)) {
-      LogParam(NPVARIANT_TO_STRING(aParam), aLog);
-      return;
-    }
-
-    NS_ERROR("Unsupported type!");
-  }
-};
-
 template <>
 struct ParamTraits<mozilla::plugins::IPCByteRange>
 {
@@ -758,7 +605,7 @@ struct ParamTraits<mozilla::plugins::IPCByteRange>
     WriteParam(aMsg, aParam.length);
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     paramType p;
     if (ReadParam(aMsg, aIter, &p.offset) &&
@@ -780,7 +627,7 @@ struct ParamTraits<NPNVariable>
     WriteParam(aMsg, int(aParam));
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     int intval;
     if (ReadParam(aMsg, aIter, &intval)) {
@@ -801,7 +648,7 @@ struct ParamTraits<NPNURLVariable>
     WriteParam(aMsg, int(aParam));
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     int intval;
     if (ReadParam(aMsg, aIter, &intval)) {
@@ -827,7 +674,7 @@ struct ParamTraits<NPCoordinateSpace>
     WriteParam(aMsg, int32_t(aParam));
   }
 
-  static bool Read(const Message* aMsg, void** aIter, paramType* aResult)
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
   {
     int32_t intval;
     if (ReadParam(aMsg, aIter, &intval)) {
@@ -845,6 +692,40 @@ struct ParamTraits<NPCoordinateSpace>
   }
 };
 
+template <>
+struct ParamTraits<mozilla::plugins::NPAudioDeviceChangeDetailsIPC>
+{
+  typedef mozilla::plugins::NPAudioDeviceChangeDetailsIPC paramType;
+
+  static void Write(Message* aMsg, const paramType& aParam)
+  {
+    WriteParam(aMsg, aParam.flow);
+    WriteParam(aMsg, aParam.role);
+    WriteParam(aMsg, aParam.defaultDevice);
+  }
+
+  static bool Read(const Message* aMsg, PickleIterator* aIter, paramType* aResult)
+  {
+    int32_t flow, role;
+    std::wstring defaultDevice;
+    if (ReadParam(aMsg, aIter, &flow) &&
+        ReadParam(aMsg, aIter, &role) &&
+        ReadParam(aMsg, aIter, &defaultDevice)) {
+      aResult->flow = flow;
+      aResult->role = role;
+      aResult->defaultDevice = defaultDevice;
+      return true;
+    }
+    return false;
+  }
+
+  static void Log(const paramType& aParam, std::wstring* aLog)
+  {
+    aLog->append(StringPrintf(L"[%d, %d, %S]", aParam.flow, aParam.role,
+                              aParam.defaultDevice.c_str()));
+  }
+};
+
 } /* namespace IPC */
 
 
