@@ -21,9 +21,6 @@
 #include "AudioManager.h"
 
 #include "nsIObserverService.h"
-#ifdef MOZ_B2G_RIL
-#include "nsIRadioInterfaceLayer.h"
-#endif
 #include "nsISettingsService.h"
 #include "nsPrintfCString.h"
 
@@ -56,10 +53,10 @@ using namespace mozilla::dom::bluetooth;
 #undef LOG
 #define LOG(args...)  __android_log_print(ANDROID_LOG_INFO, "AudioManager" , ## args)
 
-#define HEADPHONES_STATUS_HEADSET     MOZ_UTF16("headset")
-#define HEADPHONES_STATUS_HEADPHONE   MOZ_UTF16("headphone")
-#define HEADPHONES_STATUS_OFF         MOZ_UTF16("off")
-#define HEADPHONES_STATUS_UNKNOWN     MOZ_UTF16("unknown")
+#define HEADPHONES_STATUS_HEADSET     u"headset"
+#define HEADPHONES_STATUS_HEADPHONE   u"headphone"
+#define HEADPHONES_STATUS_OFF         u"off"
+#define HEADPHONES_STATUS_UNKNOWN     u"unknown"
 #define HEADPHONES_STATUS_CHANGED     "headphones-status-changed"
 #define MOZ_SETTINGS_CHANGE_ID        "mozsettings-changed"
 #define AUDIO_CHANNEL_PROCESS_CHANGED "audio-channel-process-changed"
@@ -126,12 +123,22 @@ static const uint32_t sChannelStreamTbl[NUMBER_OF_AUDIO_CHANNELS] = {
   AUDIO_STREAM_SYSTEM,          // AudioChannel::System
 };
 
-// Mappings AudioOutputProfiles to strings.
-static const nsAttrValue::EnumTable kAudioOutputProfilesTable[] = {
-  { "primary",   DEVICE_PRIMARY },
-  { "headset",   DEVICE_HEADSET },
-  { "bluetooth", DEVICE_BLUETOOTH },
-  { nullptr }
+
+struct AudioDeviceInfo {
+  /** The string the value maps to */
+  const char* tag;
+  /** The enum value that maps to this string */
+  uint32_t value;
+};
+
+// Mappings audio output devices to strings.
+static const AudioDeviceInfo kAudioDeviceInfos[] = {
+  { "earpiece",        AUDIO_DEVICE_OUT_EARPIECE },
+  { "speaker",         AUDIO_DEVICE_OUT_SPEAKER },
+  { "wired_headset",   AUDIO_DEVICE_OUT_WIRED_HEADSET },
+  { "wired_headphone", AUDIO_DEVICE_OUT_WIRED_HEADPHONE },
+  { "bt_scoheadset",   AUDIO_DEVICE_OUT_BLUETOOTH_SCO_HEADSET },
+  { "bt_a2dp",         AUDIO_DEVICE_OUT_BLUETOOTH_A2DP },
 };
 
 static const int kBtSampleRate = 8000;
@@ -164,15 +171,15 @@ static const VolumeData gVolumeData[] = {
   {"audio.volume.bt_sco",       AUDIO_STREAM_BLUETOOTH_SCO}
 };
 
-class RunnableCallTask : public Task
+class RunnableCallTask : public Runnable
 {
 public:
   explicit RunnableCallTask(nsIRunnable* aRunnable)
     : mRunnable(aRunnable) {}
 
-  void Run() override
+  NS_IMETHOD Run() override
   {
-    mRunnable->Run();
+    return mRunnable->Run();
   }
 protected:
   nsCOMPtr<nsIRunnable> mRunnable;
@@ -261,7 +268,7 @@ AudioManager::HandleAudioFlingerDied()
 
   // Enable volume change notification
   mIsVolumeInited = true;
-  mAudioOutProfileUpdated = 0;
+  mAudioOutDevicesUpdated = 0;
   MaybeUpdateVolumeSettingToDatabase(true);
 }
 
@@ -288,19 +295,20 @@ public:
     for (uint32_t idx = 0; idx < MOZ_ARRAY_LENGTH(gVolumeData); ++idx) {
       NS_ConvertASCIItoUTF16 volumeType(gVolumeData[idx].mChannelName);
       if (StringBeginsWith(aName, volumeType)) {
-        AudioOutputProfiles profile = GetProfileFromSettingName(aName);
-        MOZ_ASSERT(profile != DEVICE_ERROR);
-        int32_t stream = gVolumeData[idx].mStreamType;
-        uint32_t volIndex = aResult.isInt32() ?
-                  aResult.toInt32() : sDefaultStreamVolumeTbl[stream];
-        nsresult rv = audioManager->ValidateVolumeIndex(stream, volIndex);
-        if (NS_WARN_IF(NS_FAILED(rv))) {
-          mPromiseHolder.Reject("Error : invalid volume index.", __func__);
-          return rv;
+        uint32_t device = GetDeviceFromSettingName(aName);
+        MOZ_ASSERT(device != AUDIO_DEVICE_NONE);
+        if (aResult.isInt32()) {
+          int32_t stream = gVolumeData[idx].mStreamType;
+          uint32_t volIndex = aResult.toInt32();
+          nsresult rv = audioManager->ValidateVolumeIndex(stream, volIndex);
+          if (NS_WARN_IF(NS_FAILED(rv))) {
+            mPromiseHolder.Reject("Error : invalid volume index.", __func__);
+            return rv;
+          }
+          audioManager->SetStreamVolumeForDevice(stream, volIndex, device);
         }
 
-        audioManager->InitVolumeForProfile(profile, stream, volIndex);
-        if (++mInitCounter == DEVICE_TOTAL_NUMBER * MOZ_ARRAY_LENGTH(gVolumeData)) {
+        if (++mInitCounter == MOZ_ARRAY_LENGTH(kAudioDeviceInfos) * MOZ_ARRAY_LENGTH(gVolumeData)) {
           mPromiseHolder.Resolve(true, __func__);
         }
         return NS_OK;
@@ -319,15 +327,15 @@ public:
 protected:
   ~VolumeInitCallback() {}
 
-  AudioOutputProfiles GetProfileFromSettingName(const nsAString& aName) const
+  uint32_t GetDeviceFromSettingName(const nsAString& aName) const
   {
-    for (uint32_t idx = 0; kAudioOutputProfilesTable[idx].tag; ++idx) {
-      NS_ConvertASCIItoUTF16 profile(kAudioOutputProfilesTable[idx].tag);
-      if (StringEndsWith(aName, profile)) {
-        return static_cast<AudioOutputProfiles>(kAudioOutputProfilesTable[idx].value);
+    for (uint32_t idx = 0; idx < MOZ_ARRAY_LENGTH(kAudioDeviceInfos); ++idx) {
+      NS_ConvertASCIItoUTF16 device(kAudioDeviceInfos[idx].tag);
+      if (StringEndsWith(aName, device)) {
+        return kAudioDeviceInfos[idx].value;
       }
     }
-    return DEVICE_ERROR;
+    return AUDIO_DEVICE_NONE;
   }
 
   RefPtr<VolumeInitPromise> mPromise;
@@ -364,9 +372,10 @@ AudioManager::IsFmOutConnected()
 NS_IMPL_ISUPPORTS(AudioManager, nsIAudioManager, nsIObserver)
 
 void
-AudioManager::AudioOutProfileUpdated(AudioOutputProfiles aProfile)
+AudioManager::AudioOutDeviceUpdated(uint32_t aDevice)
 {
-  mAudioOutProfileUpdated |= (1 << aProfile);
+  MOZ_ASSERT(audio_is_output_device(aDevice));
+  mAudioOutDevicesUpdated |= aDevice;
 }
 
 void
@@ -492,7 +501,7 @@ AudioManager::HandleBluetoothStatusChanged(nsISupports* aSubject,
           self->mA2dpSwitchDone = true;
         });
       MessageLoop::current()->PostDelayedTask(
-        FROM_HERE, new RunnableCallTask(runnable), 1000);
+        MakeAndAddRef<RunnableCallTask>(runnable), 1000);
 
       mA2dpSwitchDone = false;
     } else {
@@ -584,7 +593,7 @@ AudioManager::Observe(nsISupports* aSubject,
   // To process the volume control on each volume categories according to
   // change of settings
   else if (!strcmp(aTopic, MOZ_SETTINGS_CHANGE_ID)) {
-    RootedDictionary<dom::SettingChangeNotification> setting(nsContentUtils::RootingCxForThread());
+    RootedDictionary<dom::SettingChangeNotification> setting(RootingCx());
     if (!WrappedJSToDictionary(aSubject, setting)) {
       return NS_OK;
     }
@@ -651,7 +660,8 @@ AudioManager::HandleHeadphoneSwitchEvent(const hal::SwitchEvent& aEvent)
         self->UpdateHeadsetConnectionState(hal::SWITCH_STATE_OFF);
         self->mSwitchDone = true;
     });
-    MessageLoop::current()->PostDelayedTask(FROM_HERE, new RunnableCallTask(runnable), 1000);
+    MessageLoop::current()->PostDelayedTask(
+      MakeAndAddRef<RunnableCallTask>(runnable), 1000);
     mSwitchDone = false;
   } else if (aEvent.status() != hal::SWITCH_STATE_OFF) {
     UpdateHeadsetConnectionState(aEvent.status());
@@ -672,7 +682,7 @@ AudioManager::HandleHeadphoneSwitchEvent(const hal::SwitchEvent& aEvent)
 AudioManager::AudioManager()
   : mPhoneState(PHONE_STATE_CURRENT)
   , mIsVolumeInited(false)
-  , mAudioOutProfileUpdated(0)
+  , mAudioOutDevicesUpdated(0)
   , mSwitchDone(true)
 #if defined(MOZ_B2G_BT) || ANDROID_VERSION >= 17
   , mBluetoothA2dpEnabled(false)
@@ -681,10 +691,11 @@ AudioManager::AudioManager()
   , mA2dpSwitchDone(true)
 #endif
   , mObserver(new HeadphoneSwitchObserver())
-#ifdef MOZ_B2G_RIL
-  , mMuteCallToRIL(false)
-#endif
 {
+  for (uint32_t idx = 0; idx < MOZ_ARRAY_LENGTH(kAudioDeviceInfos); ++idx) {
+    mAudioDeviceTableIdMaps.Put(kAudioDeviceInfos[idx].value, idx);
+  }
+
   AudioSystem::setErrorCallback(BinderDeadCallback);
 #if ANDROID_VERSION >= 21
   android::sp<GonkAudioPortCallback> callback = new GonkAudioPortCallback();
@@ -697,6 +708,11 @@ AudioManager::AudioManager()
       new VolumeStreamState(*this, static_cast<audio_stream_type_t>(loop));
     mStreamStates.AppendElement(streamState);
   }
+  // Initialize stream volumes with default values
+  for (int32_t streamType = 0; streamType < AUDIO_STREAM_MAX; streamType++) {
+      uint32_t volIndex = sDefaultStreamVolumeTbl[streamType];
+      SetStreamVolumeForDevice(streamType, volIndex, AUDIO_DEVICE_OUT_DEFAULT);
+  }
   UpdateCachedActiveDevicesForStreams();
 
   RegisterSwitchObserver(hal::SWITCH_HEADPHONES, mObserver);
@@ -732,13 +748,6 @@ AudioManager::AudioManager()
     NS_WARNING("Failed to add audio-channel-process-changed observer!");
   }
 
-#ifdef MOZ_B2G_RIL
-  char value[PROPERTY_VALUE_MAX];
-  property_get("ro.moz.mute.call.to_ril", value, "false");
-  if (!strcmp(value, "true")) {
-    mMuteCallToRIL = true;
-  }
-#endif
 }
 
 AudioManager::~AudioManager() {
@@ -793,13 +802,6 @@ AudioManager::GetInstance()
 NS_IMETHODIMP
 AudioManager::GetMicrophoneMuted(bool* aMicrophoneMuted)
 {
-#ifdef MOZ_B2G_RIL
-  if (mMuteCallToRIL) {
-    // Simply return cached mIsMicMuted if mute call go via RIL.
-    *aMicrophoneMuted = mIsMicMuted;
-    return NS_OK;
-  }
-#endif
 
   if (AudioSystem::isMicrophoneMuted(aMicrophoneMuted)) {
     return NS_ERROR_FAILURE;
@@ -811,15 +813,6 @@ NS_IMETHODIMP
 AudioManager::SetMicrophoneMuted(bool aMicrophoneMuted)
 {
   if (!AudioSystem::muteMicrophone(aMicrophoneMuted)) {
-#ifdef MOZ_B2G_RIL
-    if (mMuteCallToRIL) {
-      // Extra mute request to RIL for specific platform.
-      nsCOMPtr<nsIRadioInterfaceLayer> ril = do_GetService("@mozilla.org/ril;1");
-      NS_ENSURE_TRUE(ril, NS_ERROR_FAILURE);
-      ril->SetMicrophoneMuted(aMicrophoneMuted);
-      mIsMicMuted = aMicrophoneMuted;
-    }
-#endif
     return NS_OK;
   }
   return NS_ERROR_FAILURE;
@@ -895,35 +888,6 @@ AudioManager::GetForceForUse(int32_t aUsage, int32_t* aForce) {
 #endif
 }
 
-NS_IMETHODIMP
-AudioManager::GetFmRadioAudioEnabled(bool *aFmRadioAudioEnabled)
-{
-  *aFmRadioAudioEnabled = IsFmOutConnected();
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-AudioManager::SetFmRadioAudioEnabled(bool aFmRadioAudioEnabled)
-{
-  UpdateDeviceConnectionState(aFmRadioAudioEnabled,
-                              AUDIO_DEVICE_OUT_FM,
-                              NS_LITERAL_CSTRING(""));
-  // AUDIO_STREAM_FM is not used on recent gonk.
-  // AUDIO_STREAM_MUSIC is used for FM radio volume control.
-#if ANDROID_VERSION < 19
-  // sync volume with music after powering on fm radio
-  if (aFmRadioAudioEnabled) {
-    uint32_t volIndex = mStreamStates[AUDIO_STREAM_MUSIC]->GetVolumeIndex();
-    nsresult rv = mStreamStates[AUDIO_STREAM_FM]->
-      SetVolumeIndex(volIndex, AUDIO_DEVICE_OUT_FM);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return rv;
-    }
-  }
-#endif
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 AudioManager::SetAudioChannelVolume(uint32_t aChannel, uint32_t aIndex)
 {
@@ -978,31 +942,17 @@ AudioManager::ValidateVolumeIndex(int32_t aStream, uint32_t aIndex) const
 }
 
 nsresult
-AudioManager::SetStreamVolumeForProfile(AudioOutputProfiles aProfile,
-                                        int32_t aStream,
-                                        uint32_t aIndex)
+AudioManager::SetStreamVolumeForDevice(int32_t aStream,
+                                       uint32_t aIndex,
+                                       uint32_t aDevice)
 {
   if (aStream <= AUDIO_STREAM_DEFAULT || aStream >= AUDIO_STREAM_MAX) {
     return NS_ERROR_INVALID_ARG;
   }
 
   int32_t streamAlias = sStreamVolumeAliasTbl[aStream];
-  VolumeStreamState* state = mStreamStates[streamAlias].get();
-  // Rescaling of index is not necessary.
-  switch (aProfile) {
-    case DEVICE_PRIMARY:
-      state->SetVolumeIndexToAliasStreams(aIndex, AUDIO_DEVICE_OUT_SPEAKER);
-      break;
-    case DEVICE_HEADSET:
-      state->SetVolumeIndexToAliasStreams(aIndex, AUDIO_DEVICE_OUT_WIRED_HEADSET);
-      break;
-    case DEVICE_BLUETOOTH:
-      state->SetVolumeIndexToAliasStreams(aIndex, AUDIO_DEVICE_OUT_BLUETOOTH_A2DP);
-      break;
-    default:
-      break;
-  }
-  return NS_OK;
+  VolumeStreamState* streamState = mStreamStates[streamAlias].get();
+  return streamState->SetVolumeIndexToAliasStreams(aIndex, aDevice);
 }
 
 nsresult
@@ -1056,17 +1006,15 @@ AudioManager::GetStreamVolumeIndex(int32_t aStream, uint32_t *aIndex)
 }
 
 nsAutoCString
-AudioManager::AppendProfileToVolumeSetting(const char* aName, AudioOutputProfiles aProfile)
+AudioManager::AppendDeviceToVolumeSetting(const char* aName, uint32_t aDevice)
 {
   nsAutoCString topic;
   topic.Assign(aName);
-  for (uint32_t idx = 0; kAudioOutputProfilesTable[idx].tag; ++idx) {
-    if (kAudioOutputProfilesTable[idx].value == aProfile) {
-      topic.Append(".");
-      topic.Append(kAudioOutputProfilesTable[idx].tag);
-      break;
-    }
-  }
+  topic.Append(".");
+  uint32_t index = 0;
+  DebugOnly<bool> exist = mAudioDeviceTableIdMaps.Get(aDevice, &index);
+  MOZ_ASSERT(exist);
+  topic.Append(kAudioDeviceInfos[index].tag);
   return topic;
 }
 
@@ -1088,37 +1036,29 @@ AudioManager::InitVolumeFromDatabase()
   RefPtr<VolumeInitCallback> callback = new VolumeInitCallback();
   MOZ_ASSERT(callback);
   callback->GetPromise()->Then(AbstractThread::MainThread(), __func__, this,
-                               &AudioManager::InitProfileVolumeSucceeded,
-                               &AudioManager::InitProfileVolumeFailed);
+                               &AudioManager::InitDeviceVolumeSucceeded,
+                               &AudioManager::InitDeviceVolumeFailed);
 
   for (uint32_t idx = 0; idx < MOZ_ARRAY_LENGTH(gVolumeData); ++idx) {
-    for (uint32_t profile = 0; profile < DEVICE_TOTAL_NUMBER; ++profile) {
-      lock->Get(AppendProfileToVolumeSetting(gVolumeData[idx].mChannelName,
-                  static_cast<AudioOutputProfiles>(profile)).get(), callback);
+    for (uint32_t idx2 = 0; idx2 < MOZ_ARRAY_LENGTH(kAudioDeviceInfos); ++idx2) {
+      lock->Get(AppendDeviceToVolumeSetting(gVolumeData[idx].mChannelName,
+                                            kAudioDeviceInfos[idx2].value).get(),
+                callback);
     }
   }
 }
 
 void
-AudioManager::InitProfileVolumeSucceeded()
+AudioManager::InitDeviceVolumeSucceeded()
 {
   mIsVolumeInited = true;
   MaybeUpdateVolumeSettingToDatabase(true);
 }
 
 void
-AudioManager::InitProfileVolumeFailed(const char* aError)
+AudioManager::InitDeviceVolumeFailed(const char* aError)
 {
-  // Initialize stream volumes with default values
-  for (uint32_t idx = 0; idx < MOZ_ARRAY_LENGTH(gVolumeData); ++idx) {
-    for (int32_t profile = 0; profile < DEVICE_TOTAL_NUMBER; ++profile) {
-      int32_t stream = gVolumeData[idx].mStreamType;
-      uint32_t volIndex = sDefaultStreamVolumeTbl[stream];
-      InitVolumeForProfile(static_cast<AudioOutputProfiles>(profile),
-                           stream,
-                           volIndex);
-    }
-  }
+  // Default volume of AUDIO_DEVICE_OUT_DEFAULT is already set.
   mIsVolumeInited = true;
   MaybeUpdateVolumeSettingToDatabase(true);
   NS_WARNING(aError);
@@ -1137,8 +1077,7 @@ AudioManager::MaybeUpdateVolumeSettingToDatabase(bool aForce)
   }
 
   // Send events to update the Gaia volumes
-  mozilla::AutoSafeJSContext cx;
-  JS::Rooted<JS::Value> value(cx);
+  JS::Rooted<JS::Value> value(RootingCx());
   uint32_t volume = 0;
   for (uint32_t idx = 0; idx < MOZ_ARRAY_LENGTH(gVolumeData); ++idx) {
     int32_t streamType = gVolumeData[idx].mStreamType;
@@ -1153,7 +1092,7 @@ AudioManager::MaybeUpdateVolumeSettingToDatabase(bool aForce)
   }
 
   // For reducing the code dependency, Gaia doesn't need to know the
-  // profile volume, it only need to care about different volume categories.
+  // device volume, it only need to care about different volume categories.
   // However, we need to send the setting volume to the permanent database,
   // so that we can store the volume setting even if the phone reboots.
 
@@ -1165,29 +1104,21 @@ AudioManager::MaybeUpdateVolumeSettingToDatabase(bool aForce)
         continue;
     }
 
-    if (mAudioOutProfileUpdated & (1 << DEVICE_PRIMARY)) {
-      volume = streamState->GetVolumeIndex(AUDIO_DEVICE_OUT_SPEAKER);
+    uint32_t remainingDevices = mAudioOutDevicesUpdated;
+    for (uint32_t i = 0; remainingDevices != 0; i++) {
+      uint32_t device = (1 << i);
+      if ((device & remainingDevices) == 0) {
+        continue;
+      }
+      remainingDevices &= ~device;
+      if (!mAudioDeviceTableIdMaps.Get(device, nullptr)) {
+        continue;
+      }
+      volume = streamState->GetVolumeIndex(device);
       value.setInt32(volume);
-      lock->Set(AppendProfileToVolumeSetting(
-                  gVolumeData[idx].mChannelName,
-                  DEVICE_PRIMARY).get(),
-                  value, nullptr, nullptr);
-    }
-    if (mAudioOutProfileUpdated & (1 << DEVICE_HEADSET)) {
-      volume = streamState->GetVolumeIndex(AUDIO_DEVICE_OUT_WIRED_HEADSET);
-      value.setInt32(volume);
-      lock->Set(AppendProfileToVolumeSetting(
-                  gVolumeData[idx].mChannelName,
-                  DEVICE_HEADSET).get(),
-                  value, nullptr, nullptr);
-    }
-    if (mAudioOutProfileUpdated & (1 << DEVICE_BLUETOOTH)) {
-      volume = streamState->GetVolumeIndex(AUDIO_DEVICE_OUT_BLUETOOTH_A2DP);
-      value.setInt32(volume);
-      lock->Set(AppendProfileToVolumeSetting(
-                  gVolumeData[idx].mChannelName,
-                  DEVICE_BLUETOOTH).get(),
-                  value, nullptr, nullptr);
+      lock->Set(AppendDeviceToVolumeSetting(gVolumeData[idx].mChannelName,
+                                            device).get(),
+                value, nullptr, nullptr);
     }
   }
 
@@ -1197,21 +1128,8 @@ AudioManager::MaybeUpdateVolumeSettingToDatabase(bool aForce)
     mStreamStates[streamType]->ClearDevicesChanged();
     mStreamStates[streamType]->ClearVolumeIndexesChanged();
   }
-  // Clear mAudioOutProfileUpdated
-  mAudioOutProfileUpdated = 0;
-}
-
-void
-AudioManager::InitVolumeForProfile(AudioOutputProfiles aProfile,
-                                   int32_t aStreamType,
-                                   uint32_t aIndex)
-{
-  // Set volume to streams of aStreamType and devices of Profile.
-  for (int32_t streamType = 0; streamType < AUDIO_STREAM_MAX; streamType++) {
-    if (aStreamType == sStreamVolumeAliasTbl[streamType]) {
-      SetStreamVolumeForProfile(aProfile, streamType, aIndex);
-    }
-  }
+  // Clear mAudioOutDevicesUpdated
+  mAudioOutDevicesUpdated = 0;
 }
 
 void
@@ -1250,7 +1168,9 @@ AudioManager::GetDevicesForStream(int32_t aStream, bool aFromCache)
 
   return static_cast<uint32_t>(devices);
 #else
-  return AUDIO_DEVICE_OUT_DEFAULT;
+  // Per audio out device volume is not supported.
+  // Use AUDIO_DEVICE_OUT_SPEAKER just to store audio volume to DB.
+  return AUDIO_DEVICE_OUT_SPEAKER;
 #endif
 }
 
@@ -1383,24 +1303,12 @@ AudioManager::VolumeStreamState::SetVolumeIndexToActiveDevices(uint32_t aIndex)
 
   // AudioPolicyManager::setStreamVolumeIndex() set volumes of all active
   // devices for stream.
-  nsresult rv = SetVolumeIndexToAliasDevices(aIndex, device);
+  nsresult rv;
+  rv = SetVolumeIndexToConsistentDeviceIfNeeded(aIndex, device);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  // Workaround to make audio volume control consisitent.
-  // Active devices of AUDIO_STREAM_NOTIFICATION are affected by
-  // AUDIO_STREAM_MUSIC's activity. It makes audio volume control inconsistent.
-  // See Bug 1196724
-  if (device != AUDIO_DEVICE_OUT_SPEAKER &&
-      mStreamType == AUDIO_STREAM_NOTIFICATION) {
-      // Rescaling of index is not necessary.
-      rv = SetVolumeIndexToAliasDevices(aIndex, AUDIO_DEVICE_OUT_SPEAKER);
-      if (NS_WARN_IF(NS_FAILED(rv))) {
-        return rv;
-      }
-  }
-
   return NS_OK;
 }
 
@@ -1414,7 +1322,8 @@ AudioManager::VolumeStreamState::SetVolumeIndexToAliasStreams(uint32_t aIndex,
     // No update
     return NS_OK;
   }
-  nsresult rv = SetVolumeIndexToAliasDevices(aIndex, aDevice);
+
+  nsresult rv = SetVolumeIndexToConsistentDeviceIfNeeded(aIndex, aDevice);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -1430,66 +1339,29 @@ AudioManager::VolumeStreamState::SetVolumeIndexToAliasStreams(uint32_t aIndex,
       }
     }
   }
+
   return NS_OK;
 }
 
 nsresult
-AudioManager::VolumeStreamState::SetVolumeIndexToAliasDevices(uint32_t aIndex,
-                                                              uint32_t aDevice)
+AudioManager::VolumeStreamState::SetVolumeIndexToConsistentDeviceIfNeeded(uint32_t aIndex, uint32_t aDevice)
 {
-#if ANDROID_VERSION >= 17
-  nsresult rv = NS_ERROR_FAILURE;
-  switch (aDevice) {
-    case AUDIO_DEVICE_OUT_EARPIECE:
-    case AUDIO_DEVICE_OUT_SPEAKER:
-      // Apply volume index of DEVICE_PRIMARY devices
-      rv = SetVolumeIndex(aIndex, AUDIO_DEVICE_OUT_EARPIECE);
-      if (NS_WARN_IF(NS_FAILED(rv))) {
-        return rv;
-      }
-      rv = SetVolumeIndex(aIndex, AUDIO_DEVICE_OUT_SPEAKER);
-      if (NS_WARN_IF(NS_FAILED(rv))) {
-        return rv;
-      }
-      mManager.AudioOutProfileUpdated(DEVICE_PRIMARY);
-      break;
-    case AUDIO_DEVICE_OUT_WIRED_HEADSET:
-    case AUDIO_DEVICE_OUT_WIRED_HEADPHONE:
-      // Apply volume index of DEVICE_HEADSET devices
-      rv = SetVolumeIndex(aIndex, AUDIO_DEVICE_OUT_WIRED_HEADSET);
-      if (NS_WARN_IF(NS_FAILED(rv))) {
-        return rv;
-      }
-      rv = SetVolumeIndex(aIndex, AUDIO_DEVICE_OUT_WIRED_HEADPHONE);
-      if (NS_WARN_IF(NS_FAILED(rv))) {
-        return rv;
-      }
-      mManager.AudioOutProfileUpdated(DEVICE_HEADSET);
-      break;
-    case AUDIO_DEVICE_OUT_BLUETOOTH_SCO_HEADSET:
-    case AUDIO_DEVICE_OUT_BLUETOOTH_A2DP:
-      // Apply volume index of DEVICE_BLUETOOTH devices
-      rv = SetVolumeIndex(aIndex, AUDIO_DEVICE_OUT_BLUETOOTH_SCO_HEADSET);
-      if (NS_WARN_IF(NS_FAILED(rv))) {
-        return rv;
-      }
-      rv = SetVolumeIndex(aIndex, AUDIO_DEVICE_OUT_BLUETOOTH_A2DP);
-      if (NS_WARN_IF(NS_FAILED(rv))) {
-        return rv;
-      }
-      mManager.AudioOutProfileUpdated(DEVICE_BLUETOOTH);
-      break;
-    default:
-      rv = SetVolumeIndex(aIndex, aDevice);
-      if (NS_WARN_IF(NS_FAILED(rv))) {
-        return rv;
-      }
-      break;
+  nsresult rv;
+  if (aDevice == AUDIO_DEVICE_OUT_SPEAKER || aDevice == AUDIO_DEVICE_OUT_EARPIECE) {
+    // Set AUDIO_DEVICE_OUT_SPEAKER and AUDIO_DEVICE_OUT_EARPIECE to same volume.
+    rv = SetVolumeIndex(aIndex, AUDIO_DEVICE_OUT_SPEAKER);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+    rv = SetVolumeIndex(aIndex, AUDIO_DEVICE_OUT_EARPIECE);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+  } else {
+    // No alias device
+    rv = SetVolumeIndex(aIndex, aDevice);
   }
-#else
-  SetVolumeIndex(aIndex, aDevice);
-#endif
-  return NS_OK;
+  return rv;
 }
 
 nsresult
@@ -1502,6 +1374,7 @@ AudioManager::VolumeStreamState::SetVolumeIndex(uint32_t aIndex,
   if (aUpdateCache) {
     mVolumeIndexes.Put(aDevice, aIndex);
     mIsVolumeIndexesChanged = true;
+    mManager.AudioOutDeviceUpdated(aDevice);
   }
 
   rv = AudioSystem::setStreamVolumeIndex(
@@ -1511,8 +1384,11 @@ AudioManager::VolumeStreamState::SetVolumeIndex(uint32_t aIndex,
   return rv ? NS_ERROR_FAILURE : NS_OK;
 #else
   if (aUpdateCache) {
-    mVolumeIndexes.Put(AUDIO_DEVICE_OUT_DEFAULT, aIndex);
+    // Per audio out device volume is not supported.
+    // Use AUDIO_DEVICE_OUT_SPEAKER just to store audio volume to DB.
+    mVolumeIndexes.Put(AUDIO_DEVICE_OUT_SPEAKER, aIndex);
     mIsVolumeIndexesChanged = true;
+    mManager.AudioOutDeviceUpdated(AUDIO_DEVICE_OUT_SPEAKER);
   }
   rv = AudioSystem::setStreamVolumeIndex(
          static_cast<audio_stream_type_t>(mStreamType),
