@@ -455,18 +455,6 @@ RadioInterfaceLayer.prototype = {
     return this.radioInterfaces[clientId];
   },
 
-  getClientIdForEmergencyCall: function() {
-    // Select the client with sim card first.
-    for (let cid = 0; cid < this.numRadioInterfaces; ++cid) {
-      if (this.getRadioInterface(cid).isCardPresent()) {
-        return cid;
-      }
-    }
-
-    // Use the defualt client if no card presents.
-    return HW_DEFAULT_CLIENT_ID;
-  },
-
   setMicrophoneMuted: function(muted) {
     for (let clientId = 0; clientId < this.numRadioInterfaces; clientId++) {
       let radioInterface = this.radioInterfaces[clientId];
