@@ -1,5 +1,7 @@
+// Copyright (C) 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*************************************************************************
-* Copyright (c) 1997-2013, International Business Machines Corporation
+* Copyright (c) 1997-2016, International Business Machines Corporation
 * and others. All Rights Reserved.
 **************************************************************************
 *
@@ -66,8 +68,8 @@ class StringEnumeration;
  * TimeZone *tz = TimeZone::createTimeZone("America/Los_Angeles");
  * </pre>
  * \htmlonly</blockquote>\endhtmlonly
- * You can use <code>getAvailableIDs</code> method to iterate through
- * all the supported time zone IDs, or getCanonicalID method to check
+ * You can use the <code>createEnumeration</code> method to iterate through
+ * all the supported time zone IDs, or the <code>getCanonicalID</code> method to check
  * if a time zone ID is supported or not.  You can then choose a
  * supported ID to get a <code>TimeZone</code>.
  * If the time zone you want is not represented by one of the
@@ -273,6 +275,21 @@ public:
     static const UnicodeString U_EXPORT2 getEquivalentID(const UnicodeString& id,
                                                int32_t index);
 
+    /**
+     * Creates an instance of TimeZone detected from the current host
+     * system configuration. Note that ICU4C does not change the default
+     * time zone unless TimeZone::adoptDefault(TimeZone*) or
+     * TimeZone::setDefault(const TimeZone&) is explicitly called by a
+     * user. This method does not update the current ICU's default,
+     * and may return a different TimeZone from the one returned by
+     * TimeZone::createDefault().
+     *
+     * @return  A new instance of TimeZone detected from the current host system
+     *          configuration.
+     * @stable ICU 55
+     */
+    static TimeZone* U_EXPORT2 detectHostTimeZone();
+
     /**
      * Creates a new copy of the default TimeZone for this host. Unless the default time
      * zone has already been set using adoptDefault() or setDefault(), the default is
@@ -287,6 +304,9 @@ public:
      */
     static TimeZone* U_EXPORT2 createDefault(void);
 
+#define ICU_TZ_HAS_RECREATE_DEFAULT
+    static void U_EXPORT2 recreateDefault();
+
     /**
      * Sets the default time zone (i.e., what's returned by createDefault()) to be the
      * specified time zone.  If NULL is specified for the time zone, the default time
@@ -330,7 +350,7 @@ public:
      * @param id            The input time zone ID to be canonicalized.
      * @param canonicalID   Receives the canonical system time zone ID
      *                      or the custom time zone ID in normalized format.
-     * @param status        Recevies the status.  When the given time zone ID
+     * @param status        Receives the status.  When the given time zone ID
      *                      is neither a known system time zone ID nor a
      *                      valid custom time zone ID, U_ILLEGAL_ARGUMENT_ERROR
      *                      is set.
@@ -348,7 +368,7 @@ public:
      *                      or the custom time zone ID in normalized format.
      * @param isSystemID    Receives if the given ID is a known system
      *                      time zone ID.
-     * @param status        Recevies the status.  When the given time zone ID
+     * @param status        Receives the status.  When the given time zone ID
      *                      is neither a known system time zone ID nor a
      *                      valid custom time zone ID, U_ILLEGAL_ARGUMENT_ERROR
      *                      is set.
@@ -358,7 +378,6 @@ public:
     static UnicodeString& U_EXPORT2 getCanonicalID(const UnicodeString& id,
         UnicodeString& canonicalID, UBool& isSystemID, UErrorCode& status);
 
-#ifndef U_HIDE_DRAFT_API
     /**
     * Converts a system time zone ID to an equivalent Windows time zone ID. For example,
     * Windows time zone ID "Pacific Standard Time" is returned for input "America/Los_Angeles".
@@ -379,7 +398,7 @@ public:
     * @return          A reference to the result (<code>winid</code>).
     * @see getIDForWindowsID
     *
-    * @draft ICU 52
+    * @stable ICU 52
     */
     static UnicodeString& U_EXPORT2 getWindowsID(const UnicodeString& id,
         UnicodeString& winid, UErrorCode& status);
@@ -408,13 +427,11 @@ public:
     * @return          A reference to the result (<code>id</code>).
     * @see getWindowsID
     *
-    * @draft ICU 52
+    * @stable ICU 52
     */
     static UnicodeString& U_EXPORT2 getIDForWindowsID(const UnicodeString& winid, const char* region,
         UnicodeString& id, UErrorCode& status);
 
-#endif /* U_HIDE_DRAFT_API */
-
     /**
      * Returns true if the two TimeZones are equal.  (The TimeZone version only compares
      * IDs, but subclasses are expected to also compare the fields they add.)
@@ -612,8 +629,8 @@ public:
      * in the default locale.
      * This method returns the long name, not including daylight savings.
      * If the display name is not available for the locale,
-     * then this method returns a string in the format
-     * <code>GMT[+-]hh:mm</code>.
+     * then this method returns a string in the localized GMT offset format
+     * such as <code>GMT[+-]HH:mm</code>.
      * @param result the human-readable name of this time zone in the default locale.
      * @return       A reference to 'result'.
      * @stable ICU 2.0
@@ -625,8 +642,8 @@ public:
      * in the specified locale.
      * This method returns the long name, not including daylight savings.
      * If the display name is not available for the locale,
-     * then this method returns a string in the format
-     * <code>GMT[+-]hh:mm</code>.
+     * then this method returns a string in the localized GMT offset format
+     * such as <code>GMT[+-]HH:mm</code>.
      * @param locale the locale in which to supply the display name.
      * @param result the human-readable name of this time zone in the given locale
      *               or in the default locale if the given locale is not recognized.
@@ -639,8 +656,8 @@ public:
      * Returns a name of this time zone suitable for presentation to the user
      * in the default locale.
      * If the display name is not available for the locale,
-     * then this method returns a string in the format
-     * <code>GMT[+-]hh:mm</code>.
+     * then this method returns a string in the localized GMT offset format
+     * such as <code>GMT[+-]HH:mm</code>.
      * @param daylight if true, return the daylight savings name.
      * @param style
      * @param result the human-readable name of this time zone in the default locale.
@@ -653,8 +670,8 @@ public:
      * Returns a name of this time zone suitable for presentation to the user
      * in the specified locale.
      * If the display name is not available for the locale,
-     * then this method returns a string in the format
-     * <code>GMT[+-]hh:mm</code>.
+     * then this method returns a string in the localized GMT offset format
+     * such as <code>GMT[+-]HH:mm</code>.
      * @param daylight if true, return the daylight savings name.
      * @param style
      * @param locale the locale in which to supply the display name.
