@@ -6,14 +6,15 @@
 #include "ScrollbarActivity.h"
 #include "nsIScrollbarMediator.h"
 #include "nsIContent.h"
+#include "nsICSSDeclaration.h"
 #include "nsIDOMEvent.h"
-#include "nsIDOMElementCSSInlineStyle.h"
 #include "nsIDOMCSSStyleDeclaration.h"
 #include "nsIFrame.h"
 #include "nsContentUtils.h"
 #include "nsAString.h"
 #include "nsQueryFrame.h"
 #include "nsComponentManagerUtils.h"
+#include "nsStyledElement.h"
 #include "mozilla/LookAndFeel.h"
 #include "mozilla/Preferences.h"
 
@@ -85,7 +86,12 @@ ScrollbarActivity::ActivityStarted()
 void
 ScrollbarActivity::ActivityStopped()
 {
-  NS_ASSERTION(IsActivityOngoing(), "activity stopped while none was going on");
+  if (!IsActivityOngoing()) {
+    // This can happen if there was a frame reconstruction while the activity
+    // was ongoing. In this case we just do nothing. We should probably handle
+    // this case better.
+    return;
+  }
   NS_ASSERTION(mIsActive, "need to be active during activity");
   NS_ASSERTION(!mIsFading, "must not be fading during ongoing activity");
 
@@ -347,16 +353,13 @@ ScrollbarActivity::SetIsActive(bool aNewActive)
 static void
 SetOpacityOnElement(nsIContent* aContent, double aOpacity)
 {
-  nsCOMPtr<nsIDOMElementCSSInlineStyle> inlineStyleContent =
+  nsCOMPtr<nsStyledElement> inlineStyleContent =
     do_QueryInterface(aContent);
   if (inlineStyleContent) {
-    nsCOMPtr<nsIDOMCSSStyleDeclaration> decl;
-    inlineStyleContent->GetStyle(getter_AddRefs(decl));
-    if (decl) {
-      nsAutoString str;
-      str.AppendFloat(aOpacity);
-      decl->SetProperty(NS_LITERAL_STRING("opacity"), str, EmptyString());
-    }
+    nsICSSDeclaration* decl = inlineStyleContent->Style();
+    nsAutoString str;
+    str.AppendFloat(aOpacity);
+    decl->SetProperty(NS_LITERAL_STRING("opacity"), str, EmptyString());
   }
 }
 
@@ -386,15 +389,12 @@ ScrollbarActivity::UpdateOpacity(TimeStamp aTime)
 static void
 UnsetOpacityOnElement(nsIContent* aContent)
 {
-  nsCOMPtr<nsIDOMElementCSSInlineStyle> inlineStyleContent =
+  nsCOMPtr<nsStyledElement> inlineStyleContent =
     do_QueryInterface(aContent);
   if (inlineStyleContent) {
-    nsCOMPtr<nsIDOMCSSStyleDeclaration> decl;
-    inlineStyleContent->GetStyle(getter_AddRefs(decl));
-    if (decl) {
-      nsAutoString dummy;
-      decl->RemoveProperty(NS_LITERAL_STRING("opacity"), dummy);
-    }
+    nsICSSDeclaration* decl = inlineStyleContent->Style();
+    nsAutoString dummy;
+    decl->RemoveProperty(NS_LITERAL_STRING("opacity"), dummy);
   }
 }
 
