@@ -10,6 +10,7 @@
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/Event.h" // for nsIDOMEvent::InternalDOMEvent()
 #include "mozilla/gfx/PathHelpers.h"
+#include "mozilla/UniquePtr.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsPresContext.h"
@@ -20,6 +21,7 @@
 #include "nsIScriptError.h"
 #include "nsIStringBundle.h"
 #include "nsContentUtils.h"
+#include "ImageLayers.h"
 
 #ifdef ACCESSIBILITY
 #include "nsAccessibilityService.h"
@@ -44,7 +46,7 @@ public:
   void HasFocus(bool aHasFocus);
 
   nsCOMPtr<nsIContent> mArea;
-  nscoord* mCoords;
+  UniquePtr<nscoord[]> mCoords;
   int32_t mNumCoords;
   bool mHasFocus;
 };
@@ -54,7 +56,6 @@ Area::Area(nsIContent* aArea)
 {
   MOZ_COUNT_CTOR(Area);
   NS_PRECONDITION(mArea, "How did that happen?");
-  mCoords = nullptr;
   mNumCoords = 0;
   mHasFocus = false;
 }
@@ -62,7 +63,6 @@ Area::Area(nsIContent* aArea)
 Area::~Area()
 {
   MOZ_COUNT_DTOR(Area);
-  delete [] mCoords;
 }
 
 #include <stdlib.h>
@@ -103,7 +103,6 @@ void Area::ParseCoords(const nsAString& aSpec)
     char *tptr;
     char *n_str;
     int32_t i, cnt;
-    int32_t *value_list;
 
     /*
      * Nothing in an empty list
@@ -112,7 +111,7 @@ void Area::ParseCoords(const nsAString& aSpec)
     mCoords = nullptr;
     if (*cp == '\0')
     {
-      nsMemory::Free(cp);
+      free(cp);
       return;
     }
 
@@ -126,7 +125,7 @@ void Area::ParseCoords(const nsAString& aSpec)
     }
     if (*n_str == '\0')
     {
-      nsMemory::Free(cp);
+      free(cp);
       return;
     }
 
@@ -208,10 +207,10 @@ void Area::ParseCoords(const nsAString& aSpec)
     /*
      * Allocate space for the coordinate array.
      */
-    value_list = new nscoord[cnt];
+    UniquePtr<nscoord[]> value_list = MakeUnique<nscoord[]>(cnt);
     if (!value_list)
     {
-      nsMemory::Free(cp);
+      free(cp);
       return;
     }
 
@@ -252,9 +251,9 @@ void Area::ParseCoords(const nsAString& aSpec)
     }
 
     mNumCoords = cnt;
-    mCoords = value_list;
+    mCoords = Move(value_list);
 
-    nsMemory::Free(cp);
+    free(cp);
   }
 }
 
@@ -535,7 +534,8 @@ void PolyArea::Draw(nsIFrame* aFrame, DrawTarget& aDrawTarget,
         p2.y = pc->CSSPixelsToDevPixels(mCoords[i+1]);
         p1snapped = p1;
         p2snapped = p2;
-        SnapLineToDevicePixelsForStroking(p1snapped, p2snapped, aDrawTarget);
+        SnapLineToDevicePixelsForStroking(p1snapped, p2snapped, aDrawTarget,
+                                          aStrokeOptions.mLineWidth);
         aDrawTarget.StrokeLine(p1snapped, p2snapped, aColor, aStrokeOptions);
         p1 = p2;
       }
@@ -543,7 +543,8 @@ void PolyArea::Draw(nsIFrame* aFrame, DrawTarget& aDrawTarget,
       p2.y = pc->CSSPixelsToDevPixels(mCoords[1]);
       p1snapped = p1;
       p2snapped = p2;
-      SnapLineToDevicePixelsForStroking(p1snapped, p2snapped, aDrawTarget);
+      SnapLineToDevicePixelsForStroking(p1snapped, p2snapped, aDrawTarget,
+                                        aStrokeOptions.mLineWidth);
       aDrawTarget.StrokeLine(p1snapped, p2snapped, aColor, aStrokeOptions);
     }
   }
@@ -715,7 +716,7 @@ nsImageMap::FreeAreas()
   uint32_t i, n = mAreas.Length();
   for (i = 0; i < n; i++) {
     Area* area = mAreas.ElementAt(i);
-    if (area->mArea->IsInDoc()) {
+    if (area->mArea->IsInUncomposedDoc()) {
       NS_ASSERTION(area->mArea->GetPrimaryFrame() == mImageFrame,
                    "Unexpected primary frame");
 
@@ -759,27 +760,26 @@ nsImageMap::SearchForAreas(nsIContent* aParent, bool& aFoundArea,
   for (i = 0; i < n; i++) {
     nsIContent *child = aParent->GetChildAt(i);
 
-    if (child->IsHTML()) {
-      // If we haven't determined that the map element contains an
-      // <a> element yet, then look for <area>.
-      if (!aFoundAnchor && child->Tag() == nsGkAtoms::area) {
-        aFoundArea = true;
-        rv = AddArea(child);
-        NS_ENSURE_SUCCESS(rv, rv);
+    // If we haven't determined that the map element contains an
+    // <a> element yet, then look for <area>.
+    if (!aFoundAnchor && child->IsHTMLElement(nsGkAtoms::area)) {
+      aFoundArea = true;
+      rv = AddArea(child);
+      NS_ENSURE_SUCCESS(rv, rv);
 
-        // Continue to next child. This stops mContainsBlockContents from
-        // getting set. It also makes us ignore children of <area>s which
-        // is consistent with how we react to dynamic insertion of such
-        // children.
-        continue;
-      }
-      // If we haven't determined that the map element contains an
-      // <area> element yet, then look for <a>.
-      if (!aFoundArea && child->Tag() == nsGkAtoms::a) {
-        aFoundAnchor = true;
-        rv = AddArea(child);
-        NS_ENSURE_SUCCESS(rv, rv);
-      }
+      // Continue to next child. This stops mContainsBlockContents from
+      // getting set. It also makes us ignore children of <area>s which
+      // is consistent with how we react to dynamic insertion of such
+      // children.
+      continue;
+    }
+
+    // If we haven't determined that the map element contains an
+    // <area> element yet, then look for <a>.
+    if (!aFoundArea && child->IsHTMLElement(nsGkAtoms::a)) {
+      aFoundAnchor = true;
+      rv = AddArea(child);
+      NS_ENSURE_SUCCESS(rv, rv);
     }
 
     if (child->IsElement()) {
@@ -918,7 +918,8 @@ nsImageMap::AttributeChanged(nsIDocument*  aDocument,
                              dom::Element* aElement,
                              int32_t       aNameSpaceID,
                              nsIAtom*      aAttribute,
-                             int32_t       aModType)
+                             int32_t       aModType,
+                             const nsAttrValue* aOldValue)
 {
   // If the parent of the changing content node is our map then update
   // the map.  But only do this if the node is an HTML <area> or <a>
@@ -926,7 +927,7 @@ nsImageMap::AttributeChanged(nsIDocument*  aDocument,
   // are the only cases we care about.
   if ((aElement->NodeInfo()->Equals(nsGkAtoms::area) ||
        aElement->NodeInfo()->Equals(nsGkAtoms::a)) &&
-      aElement->IsHTML() &&
+      aElement->IsHTMLElement() &&
       aNameSpaceID == kNameSpaceID_None &&
       (aAttribute == nsGkAtoms::shape ||
        aAttribute == nsGkAtoms::coords)) {
