@@ -3,9 +3,17 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/*globals ContentAreaUtils */
+
 const { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;
 
 const APK_MIME_TYPE = "application/vnd.android.package-archive";
+
+const OMA_DOWNLOAD_DESCRIPTOR_MIME_TYPE = "application/vnd.oma.dd+xml";
+const OMA_DRM_MESSAGE_MIME = "application/vnd.oma.drm.message";
+const OMA_DRM_CONTENT_MIME = "application/vnd.oma.drm.content";
+const OMA_DRM_RIGHTS_MIME = "application/vnd.oma.drm.rights+wbxml";
+
 const PREF_BD_USEDOWNLOADDIR = "browser.download.useDownloadDir";
 const URI_GENERIC_ICON_DOWNLOAD = "drawable://alert_download";
 
@@ -13,30 +21,30 @@ Cu.import("resource://gre/modules/Downloads.jsm");
 Cu.import("resource://gre/modules/FileUtils.jsm");
 Cu.import("resource://gre/modules/HelperApps.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
+Cu.import("resource://gre/modules/NetUtil.jsm");
 Cu.import("resource://gre/modules/Task.jsm");
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 
+XPCOMUtils.defineLazyModuleGetter(this, "RuntimePermissions", "resource://gre/modules/RuntimePermissions.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "Messaging", "resource://gre/modules/Messaging.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "Snackbars", "resource://gre/modules/Snackbars.jsm");
+
 // -----------------------------------------------------------------------
 // HelperApp Launcher Dialog
 // -----------------------------------------------------------------------
 
+XPCOMUtils.defineLazyGetter(this, "ContentAreaUtils", function() {
+  let ContentAreaUtils = {};
+  Services.scriptloader.loadSubScript("chrome://global/content/contentAreaUtils.js", ContentAreaUtils);
+  return ContentAreaUtils;
+});
+
 function HelperAppLauncherDialog() { }
 
 HelperAppLauncherDialog.prototype = {
   classID: Components.ID("{e9d277a0-268a-4ec2-bb8c-10fdf3e44611}"),
   QueryInterface: XPCOMUtils.generateQI([Ci.nsIHelperAppLauncherDialog]),
 
-  getNativeWindow: function () {
-    try {
-      let win = Services.wm.getMostRecentWindow("navigator:browser");
-      if (win && win.NativeWindow) {
-        return win.NativeWindow;
-      }
-    } catch (e) {
-    }
-    return null;
-  },
-
   /**
    * Returns false if `url` represents a local or special URL that we don't
    * wish to ever download.
@@ -55,49 +63,23 @@ HelperAppLauncherDialog.prototype = {
     if (url.schemeIs("chrome") ||
         url.schemeIs("jar") ||
         url.schemeIs("resource") ||
-        url.schemeIs("wyciwyg")) {
+        url.schemeIs("wyciwyg") ||
+        url.schemeIs("file")) {
       return false;
     }
 
     // For all other URIs, try to resolve them to an inner URI, and check that.
     if (!alreadyResolved) {
-      let ioSvc = Cc["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
-      let innerURI = ioSvc.newChannelFromURI2(url,
-                                              null,      // aLoadingNode
-                                              Services.scriptSecurityManager.getSystemPrincipal(),
-                                              null,      // aTriggeringPrincipal
-                                              Ci.nsILoadInfo.SEC_NORMAL,
-                                              Ci.nsIContentPolicy.TYPE_OTHER).URI;
+      let innerURI = NetUtil.newChannel({
+        uri: url,
+        loadUsingSystemPrincipal: true
+      }).URI;
+
       if (!url.equals(innerURI)) {
         return this._canDownload(innerURI, true);
       }
     }
 
-    if (url.schemeIs("file")) {
-      // If it's in our app directory or profile directory, we never ever
-      // want to do anything with it, including saving to disk or passing the
-      // file to another application.
-      let file = url.QueryInterface(Ci.nsIFileURL).file;
-
-      // Normalize the nsILocalFile in-place. This will ensure that paths
-      // can be correctly compared via `contains`, below.
-      file.normalize();
-
-      // TODO: pref blacklist?
-
-      let appRoot = FileUtils.getFile("XREExeF", []);
-      if (appRoot.contains(file, true)) {
-        return false;
-      }
-
-      let profileRoot = FileUtils.getFile("ProfD", []);
-      if (profileRoot.contains(file, true)) {
-        return false;
-      }
-
-      return true;
-    }
-
     // Anything else is fine to download.
     return true;
   },
@@ -110,25 +92,60 @@ HelperAppLauncherDialog.prototype = {
     let mimeType = this._getMimeTypeFromLauncher(launcher);
 
     // Straight equality: nsIMIMEInfo normalizes.
-    return APK_MIME_TYPE == mimeType;
+    return APK_MIME_TYPE == mimeType || OMA_DOWNLOAD_DESCRIPTOR_MIME_TYPE == mimeType;
+  },
+
+  /**
+   * Returns true if `launcher` represents a download for which we wish to
+   * offer a "Save to disk" option.
+   */
+  _shouldAddSaveToDiskIntent: function(launcher) {
+      let mimeType = this._getMimeTypeFromLauncher(launcher);
+
+      // We can't handle OMA downloads. So don't even try. (Bug 1219078)
+      return mimeType != OMA_DOWNLOAD_DESCRIPTOR_MIME_TYPE;
+  },
+
+  /**
+   * Returns true if `launcher`represents a download that should not be handled by IceCat
+   * or a third-party app and instead be forwarded to Android's download manager.
+   */
+  _shouldForwardToAndroidDownloadManager: function(aLauncher) {
+    let forwardDownload = Services.prefs.getBoolPref('browser.download.forward_oma_android_download_manager');
+    if (!forwardDownload) {
+      return false;
+    }
+
+    let mimeType = aLauncher.MIMEInfo.MIMEType;
+    if (!mimeType) {
+      mimeType = ContentAreaUtils.getMIMETypeForURI(aLauncher.source) || "";
+    }
+
+    return [
+      OMA_DOWNLOAD_DESCRIPTOR_MIME_TYPE,
+      OMA_DRM_MESSAGE_MIME,
+      OMA_DRM_CONTENT_MIME,
+      OMA_DRM_RIGHTS_MIME
+    ].indexOf(mimeType) != -1;
   },
 
   show: function hald_show(aLauncher, aContext, aReason) {
     if (!this._canDownload(aLauncher.source)) {
-      aLauncher.cancel(Cr.NS_BINDING_ABORTED);
-
-      let win = this.getNativeWindow();
-      if (!win) {
-        // Oops.
-        Services.console.logStringMessage("Refusing download, but can't show a toast.");
-        return;
-      }
-
-      Services.console.logStringMessage("Refusing download of non-downloadable file.");
-      let bundle = Services.strings.createBundle("chrome://browser/locale/handling.properties");
-      let failedText = bundle.GetStringFromName("download.blocked");
-      win.toast.show(failedText, "long");
+      this._refuseDownload(aLauncher);
+      return;
+    }
 
+    if (this._shouldForwardToAndroidDownloadManager(aLauncher)) {
+      Task.spawn(function* () {
+        try {
+          let hasPermission = yield RuntimePermissions.waitForPermissions(RuntimePermissions.WRITE_EXTERNAL_STORAGE);
+          if (hasPermission) {
+            this._downloadWithAndroidDownloadManager(aLauncher);
+            aLauncher.cancel(Cr.NS_BINDING_ABORTED);
+          }
+        } finally {
+        }
+      }.bind(this)).catch(Cu.reportError);
       return;
     }
 
@@ -139,19 +156,36 @@ HelperAppLauncherDialog.prototype = {
       mimeType: aLauncher.MIMEInfo.MIMEType,
     });
 
-    // Add a fake intent for save to disk at the top of the list.
-    apps.unshift({
-      name: bundle.GetStringFromName("helperapps.saveToDisk"),
-      packageName: "org.mozilla.gecko.Download",
-      iconUri: "drawable://icon",
-      selected: true, // Default to download for files
-      launch: function() {
-        // Reset the preferredAction here.
-        aLauncher.MIMEInfo.preferredAction = Ci.nsIMIMEInfo.saveToDisk;
-        aLauncher.saveToDisk(null, false);
-        return true;
+    if (this._shouldAddSaveToDiskIntent(aLauncher)) {
+      // Add a fake intent for save to disk at the top of the list.
+      apps.unshift({
+        name: bundle.GetStringFromName("helperapps.saveToDisk"),
+        packageName: "org.mozilla.gecko.Download",
+        iconUri: "drawable://icon",
+        selected: true, // Default to download for files
+        launch: function() {
+          // Reset the preferredAction here.
+          aLauncher.MIMEInfo.preferredAction = Ci.nsIMIMEInfo.saveToDisk;
+          aLauncher.saveToDisk(null, false);
+          return true;
+        }
+      });
+    }
+
+    // We do not handle this download and there are no apps that want to do it
+    if (apps.length === 0) {
+      this._refuseDownload(aLauncher);
+      return;
+    }
+
+    let callback = function(app) {
+      aLauncher.MIMEInfo.preferredAction = Ci.nsIMIMEInfo.useHelperApp;
+      if (!app.launch(aLauncher.source)) {
+        // Once the app is done we need to get rid of the temp file. This shouldn't
+        // get run in the saveToDisk case.
+        aLauncher.cancel(Cr.NS_BINDING_ABORTED);
       }
-    });
+    }
 
     // See if the user already marked something as the default for this mimetype,
     // and if that app is still installed.
@@ -162,18 +196,11 @@ HelperAppLauncherDialog.prototype = {
       });
 
       if (pref.length > 0) {
-        pref[0].launch(aLauncher.source);
+        callback(pref[0]);
         return;
       }
     }
 
-    let callback = function(app) {
-      aLauncher.MIMEInfo.preferredAction = Ci.nsIMIMEInfo.useHelperApp;
-      if (!app.launch(aLauncher.source)) {
-        aLauncher.cancel(Cr.NS_BINDING_ABORTED);
-      }
-    }
-
     // If there's only one choice, and we don't want to prompt, go right ahead
     // and choose that app automatically.
     if (!this._shouldPrompt(aLauncher) && (apps.length === 1)) {
@@ -187,7 +214,9 @@ HelperAppLauncherDialog.prototype = {
       buttons: [
         bundle.GetStringFromName("helperapps.alwaysUse"),
         bundle.GetStringFromName("helperapps.useJustOnce")
-      ]
+      ],
+      // Tapping an app twice should choose "Just once".
+      doubleTapButton: 1
     }, (data) => {
       if (data.button < 0) {
         return;
@@ -201,6 +230,31 @@ HelperAppLauncherDialog.prototype = {
     });
   },
 
+  _refuseDownload: function(aLauncher) {
+    aLauncher.cancel(Cr.NS_BINDING_ABORTED);
+
+    Services.console.logStringMessage("Refusing download of non-downloadable file.");
+
+    let bundle = Services.strings.createBundle("chrome://browser/locale/handling.properties");
+    let failedText = bundle.GetStringFromName("download.blocked");
+
+    Snackbars.show(failedText, Snackbars.LENGTH_LONG);
+  },
+
+  _downloadWithAndroidDownloadManager(aLauncher) {
+    let mimeType = aLauncher.MIMEInfo.MIMEType;
+    if (!mimeType) {
+      mimeType = ContentAreaUtils.getMIMETypeForURI(aLauncher.source) || "";
+    }
+
+    Messaging.sendRequest({
+      'type': 'Download:AndroidDownloadManager',
+      'uri': aLauncher.source.spec,
+      'mimeType': mimeType,
+      'filename': aLauncher.suggestedFileName
+    });
+  },
+
   _getPrefName: function getPrefName(mimetype) {
     return "browser.download.preferred." + mimetype.replace("\\", ".");
   },
@@ -241,9 +295,15 @@ HelperAppLauncherDialog.prototype = {
     Task.spawn(function* () {
       let file = null;
       try {
-        let preferredDir = yield Downloads.getPreferredDownloadsDirectory();
-        file = this.validateLeafName(new FileUtils.File(preferredDir),
-                                     aDefaultFile, aSuggestedFileExt);
+        let hasPermission = yield RuntimePermissions.waitForPermissions(RuntimePermissions.WRITE_EXTERNAL_STORAGE);
+        if (hasPermission) {
+          // If we do have the STORAGE permission then pick the public downloads directory as destination
+          // for this file. Without the permission saveDestinationAvailable(null) will be called which
+          // will effectively cancel the download.
+          let preferredDir = yield Downloads.getPreferredDownloadsDirectory();
+          file = this.validateLeafName(new FileUtils.File(preferredDir),
+                                       aDefaultFile, aSuggestedFileExt);
+        }
       } finally {
         // The file argument will be null in case any exception occurred.
         aLauncher.saveDestinationAvailable(file);
@@ -289,7 +349,7 @@ HelperAppLauncherDialog.prototype = {
           aLocalFile.leafName = aLocalFile.leafName.replace(/^(.*\()\d+\)/, "$1" + (collisionCount+1) + ")");
         }
       }
-      aLocalFile.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0600);
+      aLocalFile.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0o600);
     }
     catch (e) {
       dump("*** exception in validateLeafName: " + e + "\n");
@@ -300,7 +360,7 @@ HelperAppLauncherDialog.prototype = {
       if (aLocalFile.leafName == "" || aLocalFile.isDirectory()) {
         aLocalFile.append("unnamed");
         if (aLocalFile.exists())
-          aLocalFile.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0600);
+          aLocalFile.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0o600);
       }
     }
   },
