@@ -39,11 +39,11 @@ using namespace mozilla::net;
 //
 // To enable logging (see prlog.h for full details):
 //
-//    set NSPR_LOG_MODULES=nsFtp:5
-//    set NSPR_LOG_FILE=nspr.log
+//    set MOZ_LOG=nsFtp:5
+//    set MOZ_LOG_FILE=ftp.log
 //
-// this enables LogLevel::Debug level information and places all output in
-// the file nspr.log
+// This enables LogLevel::Debug level information and places all output in
+// the file ftp.log.
 //
 LazyLogModule gFTPLog("nsFtp");
 #undef LOG
@@ -303,8 +303,7 @@ nsFtpProtocolHandler::RemoveConnection(nsIURI *aKey, nsFtpControlConnection* *_r
         return NS_ERROR_FAILURE;
 
     // swap connection ownership
-    *_retval = ts->conn;
-    ts->conn = nullptr;
+    ts->conn.forget(_retval);
     delete ts;
 
     return NS_OK;
@@ -347,7 +346,7 @@ nsFtpProtocolHandler::InsertConnection(nsIURI *aKey, nsFtpControlConnection *aCo
         return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    NS_ADDREF(aConn);
+    // ts->conn is a RefPtr
     ts->conn = aConn;
     ts->timer = timer;
 
