@@ -79,7 +79,7 @@ VerifySignedDigest(const SignedDigest& sd,
   return Success;
 }
 
-} // unnamed namespace
+} // namespace
 
 Result
 VerifyRSAPKCS1SignedDigestNSS(const SignedDigest& sd,
@@ -204,6 +204,13 @@ RegisterErrorTable()
       "The server presented a certificate that is valid for too long." },
     { "MOZILLA_PKIX_ERROR_REQUIRED_TLS_FEATURE_MISSING",
       "A required TLS feature is missing." },
+    { "MOZILLA_PKIX_ERROR_INVALID_INTEGER_ENCODING",
+      "The server presented a certificate that contains an invalid encoding of "
+      "an integer. Common causes include negative serial numbers, negative RSA "
+      "moduli, and encodings that are longer than necessary." },
+    { "MOZILLA_PKIX_ERROR_EMPTY_ISSUER_NAME",
+      "The server presented a certificate with an empty issuer distinguished "
+      "name." },
   };
   // Note that these error strings are not localizable.
   // When these strings change, update the localization information too.
