@@ -124,8 +124,7 @@ WeaveService.prototype = {
    */
   get enabled() {
     let prefs = Services.prefs.getBranch(SYNC_PREFS_BRANCH);
-    return prefs.prefHasUserValue("username") &&
-           prefs.prefHasUserValue("clusterURL");
+    return prefs.prefHasUserValue("username");
   },
 
   observe: function (subject, topic, data) {
@@ -185,10 +184,12 @@ AboutWeaveLog.prototype = {
     channel.originalURI = aURI;
 
     // Ensure that the about page has the same privileges as a regular directory
-    // view. That way links to files can be opened.
+    // view. That way links to files can be opened. make sure we use the correct
+    // origin attributes when creating the principal for accessing the
+    // about:sync-log data.
     let ssm = Cc["@mozilla.org/scriptsecuritymanager;1"]
                 .getService(Ci.nsIScriptSecurityManager);
-    let principal = ssm.createCodebasePrincipal(uri, {});
+    let principal = ssm.createCodebasePrincipal(uri, aLoadInfo.originAttributes);
 
     channel.owner = principal;
     return channel;
