@@ -870,7 +870,7 @@ MozMillController.prototype.mouseMove = function (doc, start, dest) {
 
 /**
  * Drag an element to the specified offset on another element, firing mouse and
- * drag events. Adapted from ChromeUtils.js synthesizeDrop()
+ * drag events. Adapted from EventUtils.js synthesizeDrop()
  *
  * @deprecated Use the MozMillElement object
  *
@@ -977,7 +977,10 @@ function browserAdditions (controller) {
         return windows.map.hasPageLoaded(utils.getWindowId(win));
       }, "Timeout", timeout, aInterval);
     }
-    catch (ex if ex instanceof errors.TimeoutError) {
+    catch (ex) {
+      if (!ex instanceof errors.TimeoutError) {
+        throw ex;
+      }
       timed_out = true;
     }
     finally {
